package javax.security.auth;

import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.text.MessageFormat;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.security.PermissionCollection;
import sun.security.util.ResourcesMgr;
import java.util.Iterator;
import java.security.Principal;
import java.util.Set;
import java.security.Permission;

public final class PrivateCredentialPermission extends Permission
{
    private static final long serialVersionUID = 5284372143517237068L;
    private static final PrivateCredentialPermission.CredOwner[] EMPTY_PRINCIPALS;
    private String credentialClass;
    private Set<Principal> principals;
    private transient PrivateCredentialPermission.CredOwner[] credOwners;
    private boolean testing;
    
    PrivateCredentialPermission(String s, Set<Principal> set) {
        super(s);
        this.testing = false;
        this.credentialClass = s;
        synchronized (set) {
            if (set.size() == 0) {
                this.credOwners = PrivateCredentialPermission.EMPTY_PRINCIPALS;
            }
            else {
                this.credOwners = new PrivateCredentialPermission.CredOwner[set.size()];
                int n = 0;
                for (Principal principal : set) {
                    this.credOwners[n++] = new PrivateCredentialPermission.CredOwner(principal.getClass().getName(), principal.getName());
                }
            }
        }
    }
    
    public PrivateCredentialPermission(String name, String anotherString) {
        super(name);
        this.testing = false;
        if (!"read".equalsIgnoreCase(anotherString)) {
            throw new IllegalArgumentException(ResourcesMgr.getString("actions.can.only.be.read."));
        }
        this.init(name);
        return;
    }
    
    public String getCredentialClass() {
        return this.credentialClass;
    }
    
    public String[][] getPrincipals() {
        if (this.credOwners == null || this.credOwners.length == 0) {
            return new String[0][0];
        }
        String[][] array = new String[this.credOwners.length][2];
        for (int i = 0; i < this.credOwners.length; ++i) {
            array[i][0] = this.credOwners[i].principalClass;
            array[i][1] = this.credOwners[i].principalName;
        }
        return array;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (permission == null || !(permission instanceof PrivateCredentialPermission)) {
            return false;
        }
        PrivateCredentialPermission privateCredentialPermission = (PrivateCredentialPermission)permission;
        return this.impliesCredentialClass(this.credentialClass, privateCredentialPermission.credentialClass) && this.impliesPrincipalSet(this.credOwners, privateCredentialPermission.credOwners);
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof PrivateCredentialPermission)) {
            return false;
        }
        PrivateCredentialPermission privateCredentialPermission = (PrivateCredentialPermission)o;
        return this.implies(privateCredentialPermission) && privateCredentialPermission.implies(this);
    }
    
    @Override
    public int hashCode() {
        return this.credentialClass.hashCode();
    }
    
    @Override
    public String getActions() {
        return "read";
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return null;
    }
    
    private void init(String s) {
        if (s == null || s.trim().length() == 0) {
            throw new IllegalArgumentException("invalid empty name");
        }
        ArrayList<PrivateCredentialPermission.CredOwner> list = new ArrayList();
        StringTokenizer stringTokenizer = new StringTokenizer(s, " ", true);
        if (this.testing) {
            System.out.println("whole name = " + s);
        }
        this.credentialClass = stringTokenizer.nextToken();
        if (this.testing) {
            System.out.println("Credential Class = " + this.credentialClass);
        }
        if (!stringTokenizer.hasMoreTokens()) {
            throw new IllegalArgumentException(new MessageFormat(ResourcesMgr.getString("permission.name.name.syntax.invalid.")).format(new Object[] { s }) + ResourcesMgr.getString("Credential.Class.not.followed.by.a.Principal.Class.and.Name"));
        }
        while (stringTokenizer.hasMoreTokens()) {
            stringTokenizer.nextToken();
            String nextToken = stringTokenizer.nextToken();
            if (this.testing) {
                System.out.println("    Principal Class = " + nextToken);
            }
            if (!stringTokenizer.hasMoreTokens()) {
                throw new IllegalArgumentException(new MessageFormat(ResourcesMgr.getString("permission.name.name.syntax.invalid.")).format(new Object[] { s }) + ResourcesMgr.getString("Principal.Class.not.followed.by.a.Principal.Name"));
            }
            stringTokenizer.nextToken();
            String s2 = stringTokenizer.nextToken();
            if (!s2.startsWith("\"")) {
                throw new IllegalArgumentException(new MessageFormat(ResourcesMgr.getString("permission.name.name.syntax.invalid.")).format(new Object[] { s }) + ResourcesMgr.getString("Principal.Name.must.be.surrounded.by.quotes"));
            }
            if (!s2.endsWith("\"")) {
                while (stringTokenizer.hasMoreTokens()) {
                    s2 += stringTokenizer.nextToken();
                    if (s2.endsWith("\"")) {
                        break;
                    }
                    continue;
                }
                if (!s2.endsWith("\"")) {
                    throw new IllegalArgumentException(new MessageFormat(ResourcesMgr.getString("permission.name.name.syntax.invalid.")).format(new Object[] { s }) + ResourcesMgr.getString("Principal.Name.missing.end.quote"));
                }
            }
            if (this.testing) {
                System.out.println("\tprincipalName = '" + s2 + "'");
            }
            String substring = s2.substring(1, s2.length() - 1);
            if (nextToken.equals("*") && !substring.equals("*")) {
                throw new IllegalArgumentException(ResourcesMgr.getString("PrivateCredentialPermission.Principal.Class.can.not.be.a.wildcard.value.if.Principal.Name.is.not.a.wildcard.value"));
            }
            if (this.testing) {
                System.out.println("\tprincipalName = '" + substring + "'");
            }
            list.add(new CredOwner(nextToken, substring));
        }
        list.toArray(this.credOwners = new PrivateCredentialPermission.CredOwner[list.size()]);
        return;
    }
    
    private boolean impliesCredentialClass(String str, String s) {
        if (str == null || s == null) {
            return false;
        }
        if (this.testing) {
            System.out.println("credential class comparison: " + str + "/" + s);
        }
        return str.equals("*") || str.equals(s);
    }
    
    private boolean impliesPrincipalSet(PrivateCredentialPermission.CredOwner[] array, PrivateCredentialPermission.CredOwner[] array2) {
        if (array == null || array2 == null) {
            return false;
        }
        if (array2.length == 0) {
            return true;
        }
        if (array.length == 0) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            boolean n = false;
            int j = 0;
            while (j < array2.length) {
                if (array[i].implies(array2[j])) {
                    n = true;
                    break;
                }
                ++j;
            }
            if (n == 0) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.getName().indexOf(" ") == -1 && this.getName().indexOf("\"") == -1) {
            this.credentialClass = this.getName();
            this.credOwners = PrivateCredentialPermission.EMPTY_PRINCIPALS;
        }
        else {
            this.init(this.getName());
        }
    }
    
    static {
        EMPTY_PRINCIPALS = new PrivateCredentialPermission.CredOwner[0];
    }
    
    static class CredOwner implements Serializable
    {
        private static final long serialVersionUID = -5607449830436408266L;
        String principalClass;
        String principalName;
        
        CredOwner(String principalClass, String principalName) {
            this.principalClass = principalClass;
            this.principalName = principalName;
        }
        
        public boolean implies(Object o) {
            if (o == null || !(o instanceof PrivateCredentialPermission.CredOwner)) {
                return false;
            }
            PrivateCredentialPermission.CredOwner credOwner = (PrivateCredentialPermission.CredOwner)o;
            return (this.principalClass.equals("*") || this.principalClass.equals(credOwner.principalClass)) && (this.principalName.equals("*") || this.principalName.equals(credOwner.principalName));
        }
        
        @Override
        public String toString() {
            return new MessageFormat(ResourcesMgr.getString("CredOwner.Principal.Class.class.Principal.Name.name")).format(new Object[] { this.principalClass, this.principalName });
        }
    }
}
